// This is our first C++ example!

// History: 2/9/24 - Discussed function overloading, the reference operator and pass-by-reference.
//                   We also discovered the std::string type and the overloaded + operator
//                   that concatenates two std::strings together.

// 2/7/24 - Learned how to display messages to the screen and get user input in C++.

#include <iostream> // allows us to work with input/output operations 
#include <string>

//using namespace std; // including an entire namespace is poor practice; I'll never do this!!!

// with the using below we can now apply string without the std:: in front of it!
using std::string; // it's ok to have individual using statements - still allows for
                   // some control over name collisions

// function overloading shown below - function overloading is the ability to 
// define different functions with the same name as long as the functions have
// different types, number, and/or order of parameters
int add(int n1, int n2);
//std::string add(std::string s1, std::string s2); // original idea
std::string add(const std::string &s1, const std::string &s2);

int main(void)
{
	int n1 = 10;
	int& refn1 = n1; // the & in this case is the reference operator - we are assigning
	                 // another name/label/alias for n1 - it's called refn1. both labels
	                 // refer to the memory that was originally allocated during the declaration
	                 // of n1.

	char name[50] = "cat";

	std::cout << "result: " << add(6, 7) << std::endl;
	// modified the statement below after class, we'll discuss more in class Mon, 2/12.
	// we started with std::string types in the parameter list; see the declaration above.
	// we modified the add () to work with references to std::string types, which means
	// copies of the std::string arguments are no longer made. This results in the need
	// to convert each original char * to a std::string type and also make the parameters
	// const since we're passing-by-reference.
	std::cout << "result: " << add(std::string(name), std::string("dog")) << std::endl;

	// std::cout << "result: " << add("cat", "dog") << std::endl; // originally had add () without
	                                                              // pass-by-reference for std::string


	// Code from Wed, 2/7/24
	//int number = 0;

	//// std used below is the standard namespace - a namespace is used to organize 
	//// code into related or logical groups; help with name collisions, too;
	//// cout is an object of type std::ostream
	//std::cout << "Enter a number: "; // << is an operator known as the "stream insertion" operator
	//// cin is an object of type std::istream
	//std::cin >> number; // no longer required to format data or use the address of (&) operator
	//                    // for user input in C++ - the type of data in inferred based on 
	//                    // the variable type;
	//					// the >> is an operator known as the "stream extraction" operator

	//std::cout << "number: " << number << std::endl; // endl inserts a newline at end of stream

	return 0;
}

int add(int n1, int n2)
{
	return n1 + n2;
}

//std::string add(std::string s1, std::string s2)
//{
//	return s1 + s2;
//}

std::string add(const std::string &s1, const std::string &s2)
{
	return s1 + s2; // + in this context means concatenate two std::string types
}